Package: scider
Type: Package
Title: Spatial cell-type inter-correlation by density in R
Version: 1.8.0
Authors@R: 
    c(person(given = "Mengbo",family = "Li", 
             role = c("aut"),
             email = "li.me@wehi.edu.au",
             comment = c(ORCID = "0000-0002-9666-5810")),
      person(given = "Ning", 
             family = "Liu",
             role = c("aut"),
             email = "liu.n@wehi.edu.au",
             comment = c(ORCID = "0000-0002-9487-9305")),
      person(given = "Quoc Hoang",
             family = "Nguyen",
             role = c("aut"),
             email = "nguyen.q@wehi.edu.au",
             comment = c(ORCID = "0009-0007-2828-7567")),
      person(given = "Yunshun",
             family = "Chen",
             role = c("aut", "cre"),
             email = "yuchen@wehi.edu.au",
             comment = c(ORCID = "0000-0003-4911-5653")))
Description: scider is an user-friendly R package providing functions 
    to model the global density of cells in a slide of spatial 
    transcriptomics data. All functions in the package are built 
    based on the SpatialExperiment object, allowing integration into 
    various spatial transcriptomics-related packages from Bioconductor. 
    After modelling density, the package allows for serveral 
    downstream analysis, including colocalization analysis, 
    boundary detection analysis and differential density analysis.
biocViews: Spatial, Transcriptomics
License: GPL-3 + file LICENSE
URL: https://github.com/ChenLaboratory/scider,
        https://chenlaboratory.github.io/scider/
BugReports: https://github.com/ChenLaboratory/scider/issues
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Imports: SpatialExperiment, SummarizedExperiment, spatstat.geom,
        spatstat.explore, sf, lwgeom, SpatialPack, ggplot2, stats,
        pheatmap, plotly, shiny, igraph, janitor, knitr, methods,
        utils, isoband, S4Vectors, grDevices, dbscan, hexDensity,
        hexbin, uwot, SingleCellExperiment, BiocNeighbors, irlba
Suggests: edgeR, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 4.3)
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/scider
git_branch: RELEASE_3_22
git_last_commit: 2a5b6a6
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 06:19:28 UTC; biocbuild
Author: Mengbo Li [aut] (ORCID: <https://orcid.org/0000-0002-9666-5810>),
  Ning Liu [aut] (ORCID: <https://orcid.org/0000-0002-9487-9305>),
  Quoc Hoang Nguyen [aut] (ORCID:
    <https://orcid.org/0009-0007-2828-7567>),
  Yunshun Chen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4911-5653>)
Maintainer: Yunshun Chen <yuchen@wehi.edu.au>
